#include <amxmodx>
#include <amxmisc>
#include <InlineConfig>

#define PLUGIN "Inline config"
#define VERSION "1.00"
#define AUTHOR "SiR"

new const c_FileExt[] = ".ini"
const c_RecordSize = 101

new Trie:h_Plugins
new g_ConfigsDir[100]

// last plugin cached
new g_LastPlugin_ID
new Array:g_LastPlugin_Array
new g_LastPlugin_NumRecords

public plugin_natives()
{
	register_library("InlineConfig")
	register_native("InlineConfig_NumRecords", "_InlineConfig_NumRecords", 0)
	register_native("InlineConfig_GetRecord", "_InlineConfig_GetRecord", 0)
}

public plugin_init()
{
	register_plugin(PLUGIN, VERSION, AUTHOR)
}

init_on_demand_once()
{
	static bool:once
	if(once) return
	else once = true
	
	h_Plugins = TrieCreate()
	get_configsdir(g_ConfigsDir, charsmax(g_ConfigsDir))
	
	new filename[50]
	new pos, len
	while((pos = read_dir(g_ConfigsDir, pos, filename, charsmax(filename), len)) != 0)
	{
		if(containi(filename, "plugins-") == 0)
		{
			if(equali(c_FileExt, filename[len - charsmax(c_FileExt)]))
				parse_this_file(filename)
		}
		else
		if(equali(filename, "plugins.ini"))
			parse_this_file(filename)
	}
}

bool:is_filename_char(character)
{
	if(character >= 'a' && character <= 'z')
		return true
	
	if(character >= 'A' && character <= 'Z')
		return true
	
	if(character >= '0' && character <= '9')
		return true
	
	if(character == '_' || character == '.')
		return true
	
	return false
}

bool:is_endline_char(character)
{
	return (!character || character == '^r' || character == '^n')
}

parse_this_file(const filename[])
{
	new fullpath[200]
	formatex(fullpath, charsmax(fullpath), "%s/%s", g_ConfigsDir, filename)
	
	new h_file = fopen(fullpath, "rt")
	if(!h_file)
	{
		log_error(AMX_ERR_GENERAL, "Can't open file %s/%s", g_ConfigsDir, filename)
		return
	}
	
	new plugin_fname[40], data[c_RecordSize + 10], pos
	new Array:h_array
	while(!feof(h_file))
	{
		data[0] = 0
		fgets(h_file, data, charsmax(data))
		trim(data)
		
		pos = 0
		while(!is_endline_char(data[pos]))
			pos++
		data[pos] = 0
		
		if(!data[0])
		{
			plugin_fname[0] = 0
			continue
		}
		
		if(data[0] != ';')
		{
			plugin_fname[0] = 0
			if(is_filename_char(data[0]))
			{
				pos = 0
				while(is_filename_char(data[pos]) && pos < charsmax(plugin_fname))
				{
					plugin_fname[pos] = data[pos]
					pos++
				}
				plugin_fname[pos] = 0
			}
			continue
		}
		
		if(data[1] != '@' || data[2] == 0)
		{
			plugin_fname[0] = 0
			continue
		}
		
		if(plugin_fname[0] == 0)
			continue
		
		if(!data[2])
			continue

		if(!TrieGetCell(h_Plugins, plugin_fname, h_array))
		{
			h_array = ArrayCreate(c_RecordSize, 1)
			TrieSetCell(h_Plugins, plugin_fname, h_array)
		}
		ArrayPushString(h_array, data[2])
	}
	fclose(h_file)
}

check_cached_plugin(plugin)
{
	if(plugin != g_LastPlugin_ID)
	{
		g_LastPlugin_ID = plugin
		
		new plugin_fname[40]
		get_plugin(plugin, plugin_fname, charsmax(plugin_fname))
		
		if(!TrieGetCell(h_Plugins, plugin_fname, g_LastPlugin_Array))
			g_LastPlugin_Array = Array:0
		
		g_LastPlugin_NumRecords = g_LastPlugin_Array ? ArraySize(g_LastPlugin_Array) : 0
	}
}

// ()
public _InlineConfig_NumRecords(plugin, num_params)
{
	init_on_demand_once()
	check_cached_plugin(plugin)
	
	return g_LastPlugin_NumRecords
}

// (rec_index, output[], len)
public _InlineConfig_GetRecord(plugin, num_params)
{
	init_on_demand_once()
	check_cached_plugin(plugin)
	
	new index = get_param(1)
	if(index > g_LastPlugin_NumRecords)
	{
		log_error(AMX_ERR_GENERAL, "Invalid record index (#%d)", index)
		return
	}
	index--
	
	new rec_data[c_RecordSize]
	ArrayGetString(g_LastPlugin_Array, index, rec_data, charsmax(rec_data))
	set_string(2, rec_data, get_param(3))
}
